/*!
LTC2378_20: 20-bit 1Msps ADC

@verbatim

The LTC2378_20 are pin-compatible, 20-bit A/D converters 
with serial I/O, and an internal reference. The ADCs typically draw only TBDmA 
from a single 5V supply. The automatic nap and sleep modes benefit power sensitive applications. 

The LTC2378_20's DC performance is outstanding with a +/-TBDLSB INL specification and 
no missing codes over temperature. The signal-to-noise ratio (SNR) for the LTC2378_20 is typically TBDdB, with the internal reference. 


Example Code:

Read ADC input.

    adc_command = LTC2378_20_CH0 | LTC2378_20_UNIPOLAR_MODE | LTC2378_20_LOW_GAIN_MODE | LTC2378_20_NORMAL_MODE;     // Single-ended, CH0, unipolar, low gain, normal mode.
    LTC2378_20_read(LTC2378_20_CS, adc_command, &adc_code);   // Throws out last reading
    LTC2378_20_read(LTC2378_20_CS, adc_command, &adc_code);   // Obtains the current reading and stores to adc_code variable

    // Convert adc_code to voltage
    adc_voltage = LTC2378_20_unipolar_code_to_voltage(adc_code, LTC2378_20_lsb, LTC2378_20_offset_unipolar_code);

@endverbatim

http://www.linear.com/product/LTC2378_20

http://www.linear.com/product/LTC2378_20#demoboards

REVISION HISTORY
$Revision: 1807 $
$Date: 2013-07-29 13:06:06 -0700 (Mon, 29 Jul 2013) $

Copyright (c) 2013, Linear Technology Corp.(LTC)
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this
   list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright notice,
   this list of conditions and the following disclaimer in the documentation
   and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

The views and conclusions contained in the software and documentation are those
of the authors and should not be interpreted as representing official policies,
either expressed or implied, of Linear Technology Corp.

The Linear Technology Linduino is not affiliated with the official Arduino team.
However, the Linduino is only possible because of the Arduino team's commitment
to the open-source community.  Please, visit http://www.arduino.cc and
http://store.arduino.cc , and consider a purchase that will help fund their
ongoing work.
*/

/*! @file
    @ingroup LTC2378_20
    Header for LTC2378_20: 20-bit 1Msps ADC
*/

#ifndef LTC2378_20_H
#define LTC2378_20_H

#include <SPI.h>

//! Define the SPI CS pin
#ifndef LTC2378_CS
#define LTC2378_CS QUIKEVAL_CS
#endif

//! @name LTC2378_20 Channel Address
//! @{
// Channel Address
#define LTC2378_ADDRESS             0x00
//!@}


//! Reads the ADC and returns 18-bit data
//! @return void
void LTC2378_20_read(uint8_t cs,           //!< Chip Select Pin 
                  uint16_t adc_command,  //!< Channel address, config bits ORed together
                  uint32_t *ptr_adc_code    //!< Returns code read from ADC (from previous conversion)
                 );


//! Calculates the LTC2378_20 input voltage given the binary data and lsb weight.
//! @return Floating point voltage
float LTC2378_20_code_to_voltage(uint32_t adc_code,                   //!< Raw ADC code
                                       float LTC2378_20_lsb,                   //!< LSB value (volts)
                                       int32_t LTC2378_20_offset_code //!< Offset code
                                      );


//! Calibrate the offset and LSB voltage given two measured offset codes, and a full-scale voltage with the corresponding code.
//! @return Void
void LTC2378_20_cal_offset(uint32_t zero_code,                      //!< Code from a zero reading
                         uint32_t fs_code,                                 //!< full-scale code
                         float zero_voltage,                               //!< Measured zero voltage
                         float fs_voltage,                                 //!< Measured full-scale voltage
                         float *LTC2378_20_lsb,                               //!< Return LSB value (volts)
                         int32_t *LTC2378_20_offset_code            //!< Return Unipolar Offset code, in floating point
                        );

#endif  //  LTC2378_20_H


